import { RequestMethodType, RequestOptions, RequestParamsConfig, Response, XMLHttpRequestInterface } from "../../interface/Request";
import { Header } from "../../interface/RequestContext";
declare class XMLHttp implements XMLHttpRequestInterface {
    xhr: XMLHttpRequest | any;
    activeXObjectText: string;
    /**
     * 初始化 XMLHttpRequest 对象
     * @param url
     * @param method
     * @param body
     * @param timeout
     * @param header
     */
    initXMLHttp<T>(url: string, method: RequestMethodType, body: any, timeout: number | undefined, header: Header | undefined): Promise<T>;
    /**
     * 异步等待XMLHttpRequest的请求成功回调
     * 相关的生命周期方法也在这里被调用
     * requestContext的一些配置新也在这里面获取
     * @param requestOptions
     * @param resolve
     * @param reject
     */
    xhrPromise(requestOptions: RequestOptions, resolve: Function, reject: Function): void;
    onReadyStateChange(): boolean;
}
/**
 * 外部可传入的参数配置
 * 其他的参数待配置
 */
export declare class HttpService<T> extends XMLHttp implements RequestParamsConfig {
    constructor(requestOptions: RequestOptions);
    readonly url: string;
    readonly method: RequestMethodType;
    readonly body: any;
    readonly timeout: number | undefined;
    readonly header: Header | undefined;
    request<T>(): Promise<T>;
}
/**
 * 提供给Request装饰器的方法 也可单独使用
 * @param requestParams 装饰器的初使参数
 * @param body 请求题
 */
export declare function request<T = Response>(requestParams: RequestOptions, body?: any): Promise<T>;
export {};
